<div class="col-sm-6 col-xs-6">
  <div class="panel panel-default">
    <div class="panel-heading">{{title}}</div>
    <div class="panel-body">
      <canvas id="myChart{{id}}" width="400" height="400">
    </canvas>
      <script>
	var ctx{{id}} = document.getElementById("myChart{{id}}");
	var myChart{{id}} = new Chart(ctx{{id}}, {
	    type: 'bar',
	    data: {
		labels: ["Red", "Blue", "Yellow", "Green", "Purple", "Orange", "Brown"],
		datasets: [{
		    label: '# of Votes',
		    data: {{data}},
		    backgroundColor: [
			'rgba(255, 99, 132, 0.2)',
			'rgba(54, 162, 235, 0.2)',
			'rgba(255, 206, 86, 0.2)',
			'rgba(75, 192, 192, 0.2)',
			'rgba(153, 102, 255, 0.2)',
			'rgba(255, 159, 64, 0.2)'
		    ],
		    borderColor: [
			'rgba(255,99,132,1)',
			'rgba(54, 162, 235, 1)',
			'rgba(255, 206, 86, 1)',
			'rgba(75, 192, 192, 1)',
			'rgba(153, 102, 255, 1)',
			'rgba(255, 159, 64, 1)'
		    ],
		    borderWidth: 1
		}]
	    },
	    options: {
		scales: {
		    yAxes: [{
			ticks: {
			    beginAtZero:true
			}
		    }]
		}
	    }
	});
      </script>
    </div>
  </div>
</div>
