<!--Full panel that has 100% of the width of the window -->
<div class="col-sm-12 col-xs-12 text-center"> 
    {% if danger %} 
  <div class="panel panel-danger">
    {% else %} 
  <div class="panel panel-primary">
    {% endif %} 
    <div class="panel-heading">{{title}}</div>
    <div class="panel-body" style="max-height: 200px;overflow-y: scroll;">{{body}}</div>
  </div>
</div>
