<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <title>Enrich SDK - Skin Preview</title>
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <script src="../static/js/jquery.min.js"></script>
    <link  href="../static/bootstrap/themes/flatly/css/bootstrap.min.css" rel="stylesheet"></link>
    <script src="../static/bootstrap/js/bootstrap.min.js"></script>
    <script src="../static/js/heatmap.js"></script>
    <link  href="../static/css/app.css" rel="stylesheet"></link>
    <link  href="../static/css/font-awesome-4.7.0/css/font-awesome.min.css" rel="stylesheet"></link>
</head>
  <body>
    <nav class="navbar navbar-default">
      <div class="container-fluid">
	<div class="navbar-header">
	  <button type="button" class="navbar-toggle collapsed" data-toggle="collapse" data-target="#bs-example-navbar-collapse-1" aria-expanded="false">
            <span class="sr-only">Toggle navigation</span>
            <span class="icon-bar"></span>
            <span class="icon-bar"></span>
            <span class="icon-bar"></span>
	  </button>
	  <a class="navbar-brand" href="/"> 
	  </a> 
	</div>
	<div class="collapse navbar-collapse" id="bs-example-navbar-collapse-1">
	  <ul class="nav navbar-nav">
	    <li><a href="/"><h4 style="font-size: 160%; color: white;"><i>Enrich SDK - Skin Test</i></h4></a></li>
	  </ul>
	  <ul class="nav navbar-nav navbar-right">
	    <li><a class="btn btn-danger" href="/shutdown"><h4>Shutdown</h3></a></li> 
	  </ul>
	</div>
      </div>
    </nav>
    <div class="container" id="container">
      {% block content %} 
      {% endblock content %} 
    </div>
  </body>
</html>
