<!--Similar to hero. This is specifically meant to show a trophy-->
<div class="col-sm-3 col-xs-6 text-center">
  <div class="panel">
    <div class="panel-body">
      <span class="fa-stack fa-3x">
	<i class="fa fa-trophy fa-stack-2x"></i>
	<span class="fa fa-stack-1x" style="color:white;">
	  <span style="font-size:30px; margin-top:-15px; display:block;">
        {{rank}}
	  </span>
	</span>
      </span>	
      <br/>
      {{body}}
    </div>
  </div>
</div>
