<!--This is useful to show a list-->
<div class="col-sm-6 col-xs-6 text-left">
  <ul class="list-unstyled">
    {% for f in flags %} 
    <li><i class="fa fa-thumb-tack"></i> {{f}}</li>
    {% endfor %} 
  </ul>
</div>
