.hero-widget { text-align: center; padding-top: 20px; padding-bottom: 20px; background-color:transparent; }
.hero-widget .icon { display: block; font-size: 48px; line-height: 48px; margin-bottom: 10px; text-align: center; }
.hero-widget var { display: block; height: 36px; font-size: 36px; line-height: 36px; font-style: normal; }
.hero-widget label { font-size: 17px; }
.hero-widget .options { margin-top: 10px; }
.iwt {
    display:block;
}
@import url(https://fonts.googleapis.com/css?family=Pacifico);
.white-bg {
    background:#ffffff !important;
}
.invert {-webkit-filter: invert(100%); filter: invert(100%);}
.grayscale {-webkit-filter: grayscale(100%); filter: grayscale(100%);}
.contrast {-webkit-filter: contrast(160%); filter: contrast(160%);}
.brightness {-webkit-filter: brightness(0.25); filter: brightness(0.25);}
.spacing {
    padding-left: 15px;
    padding-right:15px;
}

.btn-xlarge {
    padding: 18px 28px;
    font-size: 28px; //change this to your desired size
    line-height: normal;
    -webkit-border-radius: 8px;
       -moz-border-radius: 8px;
            border-radius: 8px;
}
.btn-huge{
    padding-top:20px;
    padding-bottom:20px;
}

.mylarge {
    padding-right:20px;
    width: 100%;
}

/* make sidebar nav vertical */ 
@media (min-width: 768px) {
  .sidebar-nav .navbar .navbar-collapse {
    padding: 0;
    max-height: none;
  }
  .sidebar-nav .navbar ul {
    float: none;
  }
  .sidebar-nav .navbar ul:not {
    display: block;
  }
  .sidebar-nav .navbar li {
    float: none;
    display: block;
  }
  .sidebar-nav .navbar li a {
    padding-top: 12px;
    padding-bottom: 12px;
  }
}

.large-text { 
    font-size: 1.2em;
    vertical-align:middle;
}

.xlarge-text { 
    font-size: 1.5em;
    vertical-align:middle;
}

.xxlarge-text { 
    font-size: 2em;
    vertical-align:middle;
}

.top-buffer { margin-top:20px; }

.numberCircle {
    border-radius: 50%;
    behavior: url(PIE.htc); /* remove if you don't care about IE8 */

    width: 36px;
    height: 36px;
    padding: 8px;

    background: #fff;
    border: 2px solid #666;
    color: #666;
    text-align: center;

    font: 32px Arial, sans-serif;
}

.verticaltext {
    transform: rotate(-90deg);
    transform-origin: right, top;
    -ms-transform: rotate(-90deg);
    -ms-transform-origin:right, top;
    -webkit-transform: rotate(-90deg);
    -webkit-transform-origin:right, top;
    position: absolute; 
    color: #ed217c;
}

body { padding-bottom: 70px; }
.navbar-xs { min-height:36px; height: 36px; }
.navbar-xs .navbar-brand{ padding: 0px 12px;font-size: 20px;line-height: 28px; }
.navbar-xs .navbar-nav > li > a {  padding-top: 0px; padding-bottom: 0px; line-height: 28px; }

.white { 
    color: #ffffff; 
}

borderless { 
    border: 0px; 
}
